#
# Copyright (c) 2018-2019 Charles Godwin <magnum@godwin.ca>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
# This code is provided as an example of a REST API server
# run the program with --help for details of options.
import argparse
import json
import time
import traceback
from collections import OrderedDict
from datetime import datetime
from http import HTTPStatus
from http.server import BaseHTTPRequestHandler, HTTPServer

from magnum import magnum

magnumReader = None

class magServer(BaseHTTPRequestHandler):

    def log_request(self, code='-', size='-'):
        #
        # suppress OK type messges
        #
        if isinstance(code, HTTPStatus):
            code = code.value
        if code < 200 or code >= 300:
            super().log_request(code=code, size=size)

    def _set_headers(self, contenttype="text/html"):
        self.send_response(200)
        self.send_header("Content-type", contenttype)
        self.end_headers()

    def do_GET(self):
        try:
            devices = magnumReader.getDevices()
            if len(devices) != 0:
                self._set_headers(contenttype="application/json")
                data = OrderedDict()
                now = int(time.time())
                data["datetime"] = str(
                    datetime.fromtimestamp(now).astimezone())

                data["devices"] = devices
                jsonString = json.dumps(data)
                self.wfile.write(jsonString.encode("utf8"))
                return
            else:
                message = "No data avaliable"
                code = HTTPStatus.NO_CONTENT
        except:
            message = "Exception detected attempting to read network data - " + traceback.format_exc()
            code = HTTPStatus.INTERNAL_SERVER_ERROR
        self.send_error(code, message=message)


def run(server_class=HTTPServer, handler_class=magServer, addr="", port=17223):
    server_address = (addr, port)
    httpd = server_class(server_address, handler_class)
    print(f"Starting Magnum Reader on {addr}:{port}")
    httpd.serve_forever()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Magnum Network Reader")
    parser.add_argument("-d", "--device", default="/dev/ttyUSB0",
                        help="Serial device name (default: %(default)s)")
    parser.add_argument("-p", "--port", type=int, default=17223,
                        help="Port on which the server listens (default: %(default)s)")
    seldom_used = parser.add_argument_group("seldom used arguments")
    seldom_used.add_argument("-n", "--packets", default=50, type=int,
                       help="Number of packets to generate (default: %(default)s)")
    seldom_used.add_argument("-t", "--timeout", default=0.005, type=float,
                        help="Timeout for serial read (default: %(default)s)")
    seldom_used.add_argument("-l", "--listen", default="ALL",
                        help="IP address on which the server listens (default: %(default)s)")
    seldom_used.add_argument("-nc", "--nocleanup", action="store_false",
                        help="Suppress clean up of unknown packets (default: %(default)s)", dest='cleanpackets')
    args = parser.parse_args()
    if args.listen.upper() == 'ALL':
        args.listen = ''
    print("Options:{}".format(str(args).replace("Namespace(", "").replace(")", "")))
    try:
        magnumReader = magnum.Magnum(device=args.device, packets=args.packets,
                                     timeout=args.timeout, cleanpackets=args.cleanpackets)
    except Exception as e:
        print("Error detected attempting to connect to Magnum network")
        traceback.print_exc()
        exit(2)
    run(addr=args.listen, port=args.port)



#
# Copyright (c) 2018-2019 Charles Godwin <magnum@godwin.ca>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
# This code is provided as an example of a simple program to display
# device data every 60 seconds.
# run the program with --help for details of options.

import argparse
import json
import time

from magnum import magnum

parser = argparse.ArgumentParser(description="Magnum Data Extractor Example")
parser.add_argument("-d", "--device", default="/dev/ttyUSB0",
                    help="Serial device name (default: %(default)s)")
parser.add_argument("-i", "--interval", default=60, type=int, dest='interval',
                    help="Interval, in seconds between logging (default: %(default)s)")
args = parser.parse_args()
reader = magnum.Magnum(device=args.device)
while True: 
    start = time.time()
    devices = reader.getDevices()
    print(json.dumps(devices, indent=2))
    duration = time.time() - start
    delay = args.interval - duration
    if delay > 0:
        time.sleep(delay)



#
# Copyright (c) 2018-2019 Charles Godwin <magnum@godwin.ca>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
import argparse
import time
import traceback
from os.path import abspath

from magnum import magnum


class dummypackets(magnum.Magnum):
    def readPackets(self):
        line = args.filename.readline()
        # If the file is not empty keep reading one line at a time, till the file is empty
        packets = []
        while line:
            ix = line.find("=>")
            if ix >= 0:
                packets.append(bytes.fromhex(line[ix+2:].strip()))
            line = args.filename.readline()
        args.filename.close()
        return packets

parser = argparse.ArgumentParser("Magnum packet tester")
parser.add_argument("filename", type=argparse.FileType("r", encoding="UTF-8"),
                    help="File name with dummy packets" )
args = parser.parse_args()
print("Options:{}".format(str(args).replace("Namespace(", "").replace(")", "")))
dummyreader = dummypackets()
try:
    packets=dummyreader.getPackets()
    formatstring = "Length:{0:2} {1:10}=>{2}"
    for packet in packets:
        if packet[0] == magnum.UNKNOWN:
            unknown += 1
        print(formatstring.format(
            len(packet[1]), packet[0], packet[1].hex().upper()))
except:
    print("Error detected attempting to read network data - test failed")
    traceback.print_exc()
    exit(2)



#
# Copyright (c) 2018-2019 Charles Godwin <magnum@godwin.ca>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
#  This module tests the functonality of the serial interface
#  by attempting to read 50 packets of data and parsing them
#  The results are dumped to the terminal
#
import argparse
import time
import traceback

from magnum import magnum

parser = argparse.ArgumentParser(
    description="Magnum RS485 Reader Test", fromfile_prefix_chars='@', prog="Magnum Test", epilog="Use `python -m serial.tools.list_ports` to identify devices.")
parser.add_argument("-d", "--device", default="/dev/ttyUSB0",
                    help="Serial device name (default: %(default)s)")
group = parser.add_argument_group("seldom used arguments")
group.add_argument("-n", "--packets", default=50, type=int,
                   help="Number of packets to generate (default: %(default)s)")
group.add_argument("-t", "--timeout", default=0.001, type=float,
                   help="Timeout for serial read - float between 0 and 1 second (default: %(default)s)")
group.add_argument("-nc", "--nocleanup", action="store_true",
                   help="Suppress clean up of unknown packets (default: %(default)s)", dest='cleanpackets')
args = parser.parse_args()
if args.timeout < 0 or args.timeout > 1.0:
    parser.error(
        "argument -t/--timeout: must be a number (float) between 0 and 1 second. i.e. 0.005")
if args.packets < 1:
    parser.error("argument -n/--packets: must be greater than 0.")
args.cleanpackets = not args.cleanpackets
print("Options:{}".format(str(args).replace("Namespace(", "").replace(")", "")))
try:
    reader = magnum.Magnum(device=args.device, packets=args.packets,
                           timeout=args.timeout, cleanpackets=args.cleanpackets)
except:
    print("Error detected attempting to open serial device")
    traceback.print_exc()
    exit(2)
try:
    start = time.time()
    packets = reader.getPackets()
    duration = time.time() - start
    unknown = 0
    formatstring = "Length:{0:2} {1:10}=>{2}"
    for packet in packets:
        if packet[0] == magnum.UNKNOWN:
            unknown += 1
        print(formatstring.format(
            len(packet[1]), packet[0], packet[1].hex().upper()))
    format1 = "Packets:{0} of {1} with {2} UNKNOWN, in {3:2.2f} seconds"
    format2 = "Packets:{0} in {3:2.2f} seconds"
    format = format1 if unknown > 0 else format2
    print(format.format(len(packets), args.packets, unknown, duration))
except:
    print("Error detected attempting to read network data - test failed")
    traceback.print_exc()
    exit(2)



#
# Copyright (c) 2018-2019 Charles Godwin <magnum@godwin.ca>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
import serial
import time
import argparse
import traceback
from os.path import abspath

parser = argparse.ArgumentParser("Magnum mock packet writer")
parser.add_argument("--device", default="/dev/ttyUSB0",
                    help="Serial device (default: %(default)s)")
parser.add_argument("filename", type=argparse.FileType("r", encoding="UTF-8"),
                    help="File name with dummy packets" )
parser.add_argument("--timeout",  default=0.005, type=float,
                    help="Interpacket sleep time. (default: %(default)s seconds)")
parser.add_argument("--useall", dest='useall', default=False,
                    action='store_true', help="Use UNKNOWN packets. (default: %(default)s)")
args = parser.parse_args()
print("Options:{}".format(str(args).replace("Namespace(", "").replace(")", "")))
device = args.device
sleeptime = args.timeout
useall = args.useall

try:
    writer = serial.Serial(port=args.device,
                            baudrate=19200,
                            bytesize=8,
                            stopbits=serial.STOPBITS_ONE,
                            parity=serial.PARITY_NONE,
                            exclusive=True,
                            write_timeout=1.0
                            )
    
except Exception as e:
    print('Error: Failed to open commuications port, exiting')
    traceback.print_exc()
    exit(2)

line = args.filename.readline()
# If the file is not empty keep reading one line at a time, till the file is empty
packets = []
while line:
    ix = line.find("=>")
    if ix >= 0:
        if useall or line.find('UNKNOWN') == -1:
            packets.append(bytes.fromhex(line[ix+2:].strip()))
    # use realine() to read next line
    line = args.filename.readline()
args.filename.close()
print("Processing:{} records from:{} using {}".format(
    len(packets), abspath(args.filename.name), args.device))

try: 
    while True:
        for packet in packets:
            writer.write(packet)
            writer.flush()
            time.sleep(sleeptime)
except Exception as e:
    print('Error: Timeout on write, exiting')
    traceback.print_exc()
    exit()
